@@ -1,11 +1,15 @@
 import functools
+import os
 import unicodedata
 from django.core.cache import caches
 from django.conf import settings
 
 from django.contrib.auth.decorators import login_required
+from django.contrib.auth.models import AnonymousUser
 from django.utils.decorators import method_decorator
 
+from bakery.views import BuildableListView
+
 
 def normalize_unicode(u):
     """Replace non-ASCII characters with closest ASCII equivalents
@@ -14,6 +18,19 @@ def normalize_unicode(u):
     return unicodedata.normalize('NFKD', u).encode('ascii', 'ignore')
 
 
+def order_results_by(*fields):
+    """A decorator that applies an ordering to the QuerySet returned by a
+       function.
+       """
+    def decorator(f):
+        @functools.wraps(f)
+        def wrapper(*args, **kw):
+            result = f(*args, **kw)
+            return result.order_by(*fields)
+        return wrapper
+    return decorator
+
+
 def cache_result(cache_key, timeout):
     """A decorator for caching the result of a function."""
     def decorator(f):
@@ -21,8 +38,6 @@ def cache_result(cache_key, timeout):
 
         @functools.wraps(f)
         def wrapper(*args, **kw):
-            # replace this with cache.caches when we drop Django 1.6
-            # compatibility
             cache = caches[cache_name]
             result = cache.get(cache_key)
             if result is None:
@@ -60,11 +75,27 @@ class QueryTracker(object):
         return connection.queries[:]
 
 
-# XXX: Should we use Django's version for Django >= 1.9 ?
-# This should certainly go away when we drop support for
-# Django 1.8
-class LoginRequiredMixin(object):
-    '''Must be logged in'''
-    @method_decorator(login_required)
-    def dispatch(self, *args, **kwargs):
-        return super(LoginRequiredMixin, self).dispatch(*args, **kwargs)
+class PaginatedBuildableListView(BuildableListView):
+    """BuildableListView subclass that handles pagination"""
+
+    build_prefix = '.'
+
+    def build_queryset(self):
+        """Override django-bakery's build logic to fake pagination."""
+        paths = [(os.path.join(self.build_prefix, 'index.html'), {})]
+        self.request = None
+        queryset = self.get_queryset()
+        paginator = self.get_paginator(queryset, self.get_paginate_by(queryset))
+        for page in paginator.page_range:
+            paths.append(
+                (os.path.join(self.build_prefix, 'page',
+                              '%d' % page, 'index.html'), {'page': page}))
+        for build_path, kwargs in paths:
+            self.request = self.create_request(build_path)
+            # Add a user with no permissions
+            self.request.user = AnonymousUser()
+            # Fake context so views work as expected
+            self.kwargs = kwargs
+            self.prep_directory(build_path)
+            target_path = os.path.join(settings.BUILD_DIR, build_path)
+            self.build_file(target_path, self.get_content())
